package com.qmth.themis.backend.api;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.gson.Gson;
import com.qmth.themis.backend.config.DictionaryConfig;
import com.qmth.themis.backend.mongodb.entity.*;
import com.qmth.themis.backend.util.ServletUtil;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.AuthDto;
import com.qmth.themis.business.entity.*;
import com.qmth.themis.business.enums.MqEnum;
import com.qmth.themis.business.enums.RoleEnum;
import com.qmth.themis.business.enums.SystemOperationEnum;
import com.qmth.themis.business.service.*;
import com.qmth.themis.business.util.EhcacheUtil;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.business.util.SessionUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.enums.Platform;
import com.qmth.themis.common.enums.Source;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.signature.SignatureInfo;
import com.qmth.themis.common.signature.SignatureType;
import com.qmth.themis.common.util.AesUtil;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import com.qmth.themis.mq.service.MqDtoService;
import io.swagger.annotations.*;
import org.apache.commons.lang3.RandomStringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.util.*;

/**
 * @Description: 用户 前端控制器
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Api(tags = "用户Controller")
@RestController
@RequestMapping("/${prefix.url.admin}/user")
public class TBUserController {
    private final static Logger log = LoggerFactory.getLogger(TBUserController.class);

    @Resource
    TBUserService tbUserService;

    @Resource
    EhcacheService ehcacheService;

    @Resource
    DictionaryConfig dictionaryConfig;

    @Resource
    RedisUtil redisUtil;

    @Resource
    MqDtoService mqDtoService;

    @ApiOperation(value = "用户登录接口")
    @RequestMapping(value = "/login/account", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "用户信息", response = TBUser.class)})
    public Result login(@ApiParam(value = "用户信息", required = true) @RequestBody TBUser tbUser, HttpServletRequest request) throws NoSuchAlgorithmException, UnsupportedEncodingException, InvalidKeySpecException {
        if (Objects.isNull(tbUser.getLoginName()) || Objects.equals(tbUser.getLoginName(), "")) {
            throw new BusinessException(ExceptionResultEnum.LOGIN_NAME_IS_NULL);
        }
        if (Objects.isNull(tbUser.getPassword()) || Objects.equals(tbUser.getPassword(), "")) {
            throw new BusinessException(ExceptionResultEnum.PASSWORD_IS_NULL);
        }
        String loginName = tbUser.getLoginName();
        String password = tbUser.getPassword();

        QueryWrapper<TBUser> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(TBUser::getLoginName, loginName);
        TBUser user = tbUserService.getOne(wrapper);
        //用户不存在
        if (Objects.isNull(user)) {
            throw new BusinessException(ExceptionResultEnum.USER_NO);
        }
        String loginPassword = AesUtil.decryptCs7(password, Constants.AES_RULE);
        //密码错误
        String aesPassword = AesUtil.decryptCs7(user.getPassword(), Constants.AES_RULE);
        if (!Objects.equals(loginPassword, aesPassword)) {
            throw new BusinessException(ExceptionResultEnum.PASSWORD_NO);
        }
        //停用
        if (user.getEnable().intValue() == 0) {
            throw new BusinessException(ExceptionResultEnum.USER_ENABLE);
        }
        Platform platform = Platform.valueOf(ServletUtil.getRequestPlatform(request));
        String deviceId = ServletUtil.getRequestDeviceId(request);
        //添加用户鉴权缓存
        AuthDto authDto = ehcacheService.addAccountCache(user);
        //生成token
        String token = RandomStringUtils.randomAlphanumeric(32);
        //添加用户缓存
        redisUtil.setUser(user.getId(), user);
        //添加用户会话缓存
        String sessionId = SessionUtil.digest(user.getId(), authDto.getRoleEnum().name(), platform.getSource());

        Date expire = SystemConstant.getExpireTime(platform);
        TBSession tbSession = new TBSession(sessionId, String.valueOf(user.getId()), authDto.getRoleEnum().name(), platform.getSource(), platform.name(), deviceId, request.getLocalAddr(), token, expire);
        redisUtil.setUserSession(sessionId, tbSession);
        //mq发送消息start
        mqDtoService.assembleSendOneWayMsg(dictionaryConfig.mqConfigDomain().getSessionTopic(), platform.getSource(), tbSession, MqEnum.SESSION.name(), tbSession.getId(), user.getLoginName());
        mqDtoService.assembleSendOneWayMsg(dictionaryConfig.mqConfigDomain().getUserLogTopic(), Objects.equals(authDto.getRoleEnum().name(), RoleEnum.STUDENT.name()) ? dictionaryConfig.mqConfigDomain().getUserLogTopicStudentTag() : dictionaryConfig.mqConfigDomain().getUserLogTopicUserTag(), SystemOperationEnum.LOGIN, MqEnum.USER_LOG.name(), user.getId(), user.getLoginName());
        //mq发送消息end
        //测试
        String test = SignatureInfo.build(SignatureType.TOKEN, sessionId, token);
        Map<String, Object> map = new HashMap<>();
//        map.put(SystemConstant.ACCESS_TOKEN, token);
        map.put(SystemConstant.ACCESS_TOKEN, test);
        map.put(SystemConstant.ACCOUNT, user);
        map.put(SystemConstant.SESSION_ID, sessionId);
        return ResultUtil.ok(map);
    }

    @Resource
    TEExamActivityService teExamActivityService;

    @Resource
    TEExamService teExamService;

    @Resource
    TEStudentService teStudentService;

    @Resource
    TEExamStudentService teExamStudentService;

    @Resource
    TEExamCourseService teExamCourseService;

    @Resource
    TEExamPaperService teExamPaperService;

    @Value("${db.name}")
    String dbName;

    @Resource
    MongoTemplate mongoTemplate;

    @ApiOperation(value = "用户查询接口")
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    @Transactional
    public Result list() {
        String tableName = "t_e_exam_activity_test1";
        Integer count = teExamActivityService.existTable(tableName, dbName);
        log.info("count:{}", count);
        if (count == 0) {
            teExamActivityService.createNewTable(tableName);
        }
        //学生档案
        TEStudent teStudent = new TEStudent();
        teStudent.setId(Constants.idGen.next());
        teStudent.setOrgId(1L);
        teStudent.setIdentity(RandomStringUtils.randomAlphanumeric(30));
        teStudent.setPassword("123456");
        teStudent.setIdcardNumber(RandomStringUtils.randomAlphanumeric(18));
        teStudent.setMobileNumber(RandomStringUtils.randomNumeric(11));
        teStudent.setName("aaa");
        teStudent.setGender(1);
        teStudent.setBasePhotoPath("http://11111");
        teStudent.setCreateTime(new Date());
        teStudentService.save(teStudent);

        //考试批次
        TEExam teExam = new TEExam();
        teExam.setId(Constants.idGen.next());
        teExam.setOrgId(1L);
        teExam.setCode(RandomStringUtils.randomAlphanumeric(20));
        teExam.setName("123");
        teExam.setTag("test1");
        teExam.setStartTime(new Date());
        teExam.setEndTime(new Date());
        teExam.setStatus(0);
        teExam.setShortCode("123456");
        teExam.setPrepareSeconds(30);
        teExam.setMinDurationSeconds(30);
        teExam.setPreNotice("99999");
        teExam.setPreNoticeStaySeconds(30);
        teExam.setPostNotice("88888");
        teExam.setExamCount(5);
        teExam.setBreakResumeCount(3);
        teExam.setBreakExpireSeconds(30);
        teExam.setLivenessActionCount(5);
        teExam.setLivenessActionRetryCount(5);
        teExam.setEntryFaceVerify(1);
        teExam.setEntryLivenessVerify(1);
        teExam.setConstantFaceVerify(1);
        teExam.setConstantLivenessRetryCount(1);
        teExam.setConstantLivenessVerifyCount(1);
        teExam.setClientVideoPush(1);
        teExam.setClientVideoRecord(1);
        teExam.setWxappPhotoUpload(1);
        teExam.setWxappVideoPush(1);
        teExam.setWxappVideoRecord(1);
        teExam.setCameraPhotoUpload(1);
        teExam.setWxappPhotoUpload(1);
        teExam.setReexamAuditing(1);
        teExam.setSelectStrategy(1);
        teExam.setIpLimit("123");
        teExam.setShowObjectiveScore(1);
        teExam.setCreateTime(new Date());
        teExamService.save(teExam);

        //考试场次
        TEExamActivity teExamActivity = new TEExamActivity();
        teExamActivity.setId(Constants.idGen.next());
        teExamActivity.setExamId(teExam.getId());
        teExamActivity.setCode(RandomStringUtils.randomAlphanumeric(20));
        teExamActivity.setName("456");
        teExamActivity.setPrepareSeconds(30);
        teExamActivity.setMinStartTime(new Date());
        teExamActivity.setMaxStartTime(new Date());
        teExamActivity.setMaxDurationSeconds(30);
        teExamActivity.setMinDurationSeconds(30);
        teExamActivity.setMaxFinishTime(new Date());
        teExamActivity.setExamCount(1);
        teExamActivity.setBreakExpireSeconds(1);
        teExamActivity.setBreakResumeCount(1);
        teExamActivity.setEntryFaceVerify(1);
        teExamActivity.setEntryLivenessVerify(1);
        teExamActivity.setConstantFaceVerify(1);
        teExamActivity.setConstantLivenessVerifyCount(1);
        teExamActivity.setClientVideoPush(1);
        teExamActivity.setClientVideoRecord(1);
        teExamActivity.setWxappPhotoUpload(1);
        teExamActivity.setWxappVideoPush(1);
        teExamActivity.setWxappVideoRecord(1);
        teExamActivity.setCameraPhotoUpload(1);
        teExamActivityService.insertInfo(tableName, teExamActivity);

        //考试试卷
        TEExamPaper teExamPaper = new TEExamPaper();
        teExamPaper.setId(Constants.idGen.next());
        teExamPaper.setName("test1");
        teExamPaper.setTotalScore(100D);
        teExamPaper.setPaperPath("123");
        teExamPaper.setStandardAnswerPath("234");
        teExamPaper.setDecryptSecret("345");
        teExamPaper.setEncryptMode(1);
        teExamPaper.setHasVideo(1);
        teExamPaper.setNeedVoiceAnswer(1);
        teExamPaper.setCreateTime(new Date());
        teExamPaperService.save(teExamPaper);

        //考试科目
        TEExamCourse teExamCourse = new TEExamCourse();
        teExamCourse.setId(Constants.idGen.next());
        teExamCourse.setExamId(teExam.getId());
        teExamCourse.setExamActivityId(teExamActivity.getId());
        teExamCourse.setCourseCode(RandomStringUtils.randomAlphanumeric(5));
        teExamCourse.setCourseName("测试科目1");
        teExamCourse.setObjectiveShuffle(1);
        teExamCourse.setOptionShuffle(1);
        List paperIdsList = new ArrayList();
        paperIdsList.add(teExamPaper.getId());
        paperIdsList.add(0L);
        Map paperMap = new HashMap();
        paperMap.put("paperIds", paperIdsList);
        teExamCourse.setPaperIds(JacksonUtil.parseJson(paperMap));
        teExamCourseService.save(teExamCourse);

        //考生
        TEExamStudent teExamStudent = new TEExamStudent();
        teExamStudent.setId(Constants.idGen.next());
        teExamStudent.setExamId(teExam.getId());
        teExamStudent.setExamActivityId(teExamActivity.getId());
        teExamStudent.setStudentId(teStudent.getId());
        teExamStudent.setCourseCode(teExamCourse.getCourseCode());
        teExamStudent.setRoomCode("1");
        teExamStudent.setIdentity(RandomStringUtils.randomAlphanumeric(20));
        teExamStudent.setName("132");
        Map stuMap = new HashMap();
        stuMap.put("examTest1", "aaa");
        stuMap.put("examTest2", "bbb");
        teExamStudent.setParameter(JacksonUtil.parseJson(stuMap));
        teExamStudent.setLeftExamCount(1);
        teExamStudentService.save(teExamStudent);

        Gson gson = new Gson();
        MTEStudentEntity mteStudentEntity = gson.fromJson(gson.toJson(teStudent), MTEStudentEntity.class);
        MTEExamEntity mteExamEntity = gson.fromJson(gson.toJson(teExam), MTEExamEntity.class);
        MTEExamActivityEntity mteExamActivityEntity = gson.fromJson(gson.toJson(teExamActivity), MTEExamActivityEntity.class);
        MTEExamPaperEntity mteExamPaperEntity = gson.fromJson(gson.toJson(teExamPaper), MTEExamPaperEntity.class);
        MTEExamCourseEntity mteExamCourseEntity = gson.fromJson(gson.toJson(teExamCourse), MTEExamCourseEntity.class);
        MTEExamStudentEntity mteExamStudentEntity = gson.fromJson(gson.toJson(teExamStudent), MTEExamStudentEntity.class);

//        List list = Arrays.asList(mteExamPaperEntity);
//        mteExamCourseEntity.setMteExamPaperEntityList(list);

//        list = Arrays.asList(mteExamActivityEntity);
//        mteExamEntity.setMteExamActivityEntityList(list);

//        list = Arrays.asList(mteExamCourseEntity);
//        mteExamStudentEntity.setMteExamEntity(mteExamEntity);
//        mteExamStudentEntity.setMteExamActivityEntity(mteExamActivityEntity);
//        mteExamStudentEntity.setMteStudentEntity(mteStudentEntity);
//        mteExamStudentEntity.setMteExamCourseEntityList(list);

        mongoTemplate.save(mteStudentEntity);
        mongoTemplate.save(mteExamEntity);
        mongoTemplate.save(mteExamActivityEntity);
        mongoTemplate.save(mteExamPaperEntity);
        mongoTemplate.save(mteExamCourseEntity);
        mongoTemplate.save(mteExamStudentEntity);

        IPage<TEExamActivity> map = teExamActivityService.selectListPage(new Page<>(0, 10), tableName);
        log.info("mysql map:{}", JacksonUtil.parseJson(map));

        Query query = new Query();
//        query.addCriteria(Criteria.where("questionId").in(setQuestionId).andOperator(Criteria.where("examRecordDataId").is(examSessionInfo.getExamRecordDataId())));
        query.skip((1 - 1) * 10).limit(10);//设置起始数和查询条数
        List<MTEStudentEntity> mteStudentEntityList = mongoTemplate.find(query, MTEStudentEntity.class);
        log.info("mongodb mteStudentEntityList:{}", JacksonUtil.parseJson(mteStudentEntityList));
//        List<MTEStudentEntity> mteStudentEntityList = mongoTemplate.findAll(MTEStudentEntity.class);
//        log.info("mongodb mteStudentEntityList:{}", JacksonUtil.parseJson(mteStudentEntityList));

        List<MTEExamEntity> mteExamEntityList = mongoTemplate.findAll(MTEExamEntity.class);
        log.info("mongodb mteExamEntityList:{}", JacksonUtil.parseJson(mteExamEntityList));

        List<MTEExamActivityEntity> mteExamActivityEntityList = mongoTemplate.findAll(MTEExamActivityEntity.class);
        log.info("mongodb mteExamActivityEntityList:{}", JacksonUtil.parseJson(mteExamActivityEntityList));
//        List<TEExamActivity> ps = new ArrayList<>();
//        mteExamActivityEntityList.forEach(s -> {
//            TEExamActivity t = gson.fromJson(gson.toJson(s), TEExamActivity.class);
//            ps.add(t);
//        });
////      List<TEExamActivity> ps = gson.fromJson(JacksonUtil.parseJson(mteExamActivityEntityList), new TypeToken<List<TEExamActivity>>(){}.getType());
//        map.setRecords(ps);

        List<MTEExamPaperEntity> mteExamPaperEntityList = mongoTemplate.findAll(MTEExamPaperEntity.class);
        log.info("mongodb mteExamPaperEntityList:{}", JacksonUtil.parseJson(mteExamPaperEntityList));

        List<MTEExamCourseEntity> mteExamCourseEntityList = mongoTemplate.findAll(MTEExamCourseEntity.class);
        log.info("mongodb mteExamCourseEntityList:{}", JacksonUtil.parseJson(mteExamCourseEntityList));

        List<MTEExamStudentEntity> mteExamStudentEntityList = mongoTemplate.findAll(MTEExamStudentEntity.class);
        log.info("mongodb mteExamStudentEntityList:{}", JacksonUtil.parseJson(mteExamStudentEntityList));
        List<TEExamStudent> ps = new ArrayList<>();
        mteExamStudentEntityList.forEach(s -> {
            TEExamStudent t = gson.fromJson(gson.toJson(s), TEExamStudent.class);
            ps.add(t);
        });
        List<MTEExamStudentEntity> ps1 = new ArrayList<>();
        List<TEExamStudent> list1 = teExamStudentService.list();
        list1.forEach(s -> {
            MTEExamStudentEntity t = gson.fromJson(gson.toJson(s), MTEExamStudentEntity.class);
            ps1.add(t);
        });
//        teExamActivityService.dropTable(tableName);
        return ResultUtil.ok(SystemConstant.SUCCESS);
    }

    @ApiOperation(value = "登出接口")
    @RequestMapping(value = "/logout", method = RequestMethod.GET)
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    public Result logout(HttpServletRequest request) throws NoSuchAlgorithmException {
        TBUser tbUser = (TBUser) ServletUtil.getRequestAccount(request);
        TBSession tbSession = (TBSession) ServletUtil.getRequestSession(request);
        AuthDto authDto = (AuthDto) EhcacheUtil.get(SystemConstant.AUTH_CACHE, tbUser.getId());
        if (Objects.isNull(tbSession)) {
            throw new BusinessException(ExceptionResultEnum.LOGIN_NO);
        }
        redisUtil.deleteUserSession(tbSession.getId());
        //循环检查该用户下其他平台是否存在session,不存在则删除用户缓存和鉴权缓存
        boolean delete = true;
        for (Source s : Source.values()) {
            String sessionId = SessionUtil.digest(tbUser.getId(), authDto.getRoleEnum().name(), s.name());
            if (Objects.nonNull(redisUtil.getUserSession(sessionId))) {
                delete = false;
                break;
            }
        }
        if (delete) {
            redisUtil.deleteUser(tbUser.getId());
            ehcacheService.removeAccountCache(tbUser.getId());
        }
        //mq发送消息start
        mqDtoService.assembleSendOneWayMsg(dictionaryConfig.mqConfigDomain().getUserLogTopic(), Objects.equals(authDto.getRoleEnum().name(), RoleEnum.STUDENT.name()) ? dictionaryConfig.mqConfigDomain().getUserLogTopicStudentTag() : dictionaryConfig.mqConfigDomain().getUserLogTopicUserTag(), SystemOperationEnum.LOGOUT, MqEnum.USER_LOG.name(), tbUser.getId(), tbUser.getLoginName());
        //mq发送消息end
        return ResultUtil.ok(JacksonUtil.parseJson(SystemConstant.SUCCESS));
    }
}
