package com.qmth.themis.backend.mongodb.entity;

import com.baomidou.mybatisplus.annotation.TableId;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.mapping.DBRef;
import org.springframework.data.mongodb.core.mapping.Document;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * @Description: mongodb 考试批次 entity
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/6
 */
@ApiModel(value = "TEExam", description = "考试批次")
@Document(collection = "TEExam")
public class MTEExamEntity implements Serializable {
    private static final long serialVersionUID = -1688201571728312142L;

    @Id
    @ApiModelProperty(value = "主键")
    private Long id;

    @ApiModelProperty(value = "机构ID")
    private Long orgId;

    @ApiModelProperty(value = "批次编码，机构唯一")
    private String code;

    @ApiModelProperty(value = "批次名称")
    private String name;

    @ApiModelProperty(value = "批次标注，可用于多个考试批次归类")
    private String tag;

    @ApiModelProperty(value = "批次开始时间")
    private Date startTime;

    @ApiModelProperty(value = "批次结束时间")
    private Date endTime;

    @ApiModelProperty(value = "状态")
    private Integer status;

    @ApiModelProperty(value = "考试口令")
    private String shortCode;

    @ApiModelProperty(value = "提前多长时间开始候考(秒)")
    private Integer prepareSeconds;

    @ApiModelProperty(value = "最短考试时长，相当于考试冻结时间(秒)")
    private Integer minDurationSeconds;

    @ApiModelProperty(value = "考前须知")
    private String preNotice;

    @ApiModelProperty(value = "考试须知阅读时长(秒)")
    @TableId(value = "pre_notice_stay_seconds")
    private Integer preNoticeStaySeconds;

    @ApiModelProperty(value = "考后说明，针对一个场次进行设置")
    private String postNotice;

    @ApiModelProperty(value = "允许考试次数")
    private Integer examCount;

    @ApiModelProperty(value = "断点失效时间(秒)")
    private Integer breakExpireSeconds;

    @ApiModelProperty(value = "断点续考次数")
    private Integer breakResumeCount;

    @ApiModelProperty(value = "活体动作个数")
    private Integer livenessActionCount;

    @ApiModelProperty(value = "活体单个动作重试次数")
    private Integer livenessActionRetryCount;

    @ApiModelProperty(value = "是否开启/强制开考人脸识别")
    private Integer entryFaceVerify;

    @ApiModelProperty(value = "是否开启/强制开考活体检测")
    private Integer entryLivenessVerify;

    @ApiModelProperty(value = "考试过程中人脸检测是否开启")
    private Integer constantFaceVerify;

    @ApiModelProperty(value = "考试过程中随机活体验证次数")
    private Integer constantLivenessVerifyCount;

    @ApiModelProperty(value = "考试过程中随机活体重试次数")
    private Integer constantLivenessRetryCount;

    @ApiModelProperty(value = "是否开启/强制客户端视频监控")
    private Integer clientVideoPush;

    @ApiModelProperty(value = "是否开启客户端视频转录")
    private Integer clientVideoRecord;

    @ApiModelProperty(value = "是否开启/强制微信小程序监控")
    private Integer wxappVideoPush;

    @ApiModelProperty(value = "是否开启微信小程序视频转录")
    private Integer wxappVideoRecord;

    @ApiModelProperty(value = "是否允许使用摄像头拍照答题")
    private Integer cameraPhotoUpload;

    @ApiModelProperty(value = "是否允许使用微信拍照答题")
    private Integer wxappPhotoUpload;

    @ApiModelProperty(value = "重考是否审批")
    private Integer reexamAuditing;

    @ApiModelProperty(value = "多次考试记录的选择逻辑：全部阅卷后取最高分；客观分最高；最后一次提交")
    private Integer selectStrategy;

    @ApiModelProperty(value = "IP段限制")
    private String ipLimit;

    @ApiModelProperty(value = "交卷后是否显示客观得分")
    private Integer showObjectiveScore;

    @ApiModelProperty(value = "创建时间")
    private Date createTime;

    @ApiModelProperty(value = "更新时间")
    private Date updateTime;

//    @DBRef
//    private List<MTEExamActivityEntity> mteExamActivityEntityList;
//
//    public List<MTEExamActivityEntity> getMteExamActivityEntityList() {
//        return mteExamActivityEntityList;
//    }
//
//    public void setMteExamActivityEntityList(List<MTEExamActivityEntity> mteExamActivityEntityList) {
//        this.mteExamActivityEntityList = mteExamActivityEntityList;
//    }

    public static long getSerialVersionUID() {
        return serialVersionUID;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getOrgId() {
        return orgId;
    }

    public void setOrgId(Long orgId) {
        this.orgId = orgId;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getTag() {
        return tag;
    }

    public void setTag(String tag) {
        this.tag = tag;
    }

    public Date getStartTime() {
        return startTime;
    }

    public void setStartTime(Date startTime) {
        this.startTime = startTime;
    }

    public Date getEndTime() {
        return endTime;
    }

    public void setEndTime(Date endTime) {
        this.endTime = endTime;
    }

    public Integer getStatus() {
        return status;
    }

    public void setStatus(Integer status) {
        this.status = status;
    }

    public String getShortCode() {
        return shortCode;
    }

    public void setShortCode(String shortCode) {
        this.shortCode = shortCode;
    }

    public Integer getPrepareSeconds() {
        return prepareSeconds;
    }

    public void setPrepareSeconds(Integer prepareSeconds) {
        this.prepareSeconds = prepareSeconds;
    }

    public Integer getMinDurationSeconds() {
        return minDurationSeconds;
    }

    public void setMinDurationSeconds(Integer minDurationSeconds) {
        this.minDurationSeconds = minDurationSeconds;
    }

    public String getPreNotice() {
        return preNotice;
    }

    public void setPreNotice(String preNotice) {
        this.preNotice = preNotice;
    }

    public Integer getPreNoticeStaySeconds() {
        return preNoticeStaySeconds;
    }

    public void setPreNoticeStaySeconds(Integer preNoticeStaySeconds) {
        this.preNoticeStaySeconds = preNoticeStaySeconds;
    }

    public String getPostNotice() {
        return postNotice;
    }

    public void setPostNotice(String postNotice) {
        this.postNotice = postNotice;
    }

    public Integer getExamCount() {
        return examCount;
    }

    public void setExamCount(Integer examCount) {
        this.examCount = examCount;
    }

    public Integer getBreakExpireSeconds() {
        return breakExpireSeconds;
    }

    public void setBreakExpireSeconds(Integer breakExpireSeconds) {
        this.breakExpireSeconds = breakExpireSeconds;
    }

    public Integer getBreakResumeCount() {
        return breakResumeCount;
    }

    public void setBreakResumeCount(Integer breakResumeCount) {
        this.breakResumeCount = breakResumeCount;
    }

    public Integer getLivenessActionCount() {
        return livenessActionCount;
    }

    public void setLivenessActionCount(Integer livenessActionCount) {
        this.livenessActionCount = livenessActionCount;
    }

    public Integer getLivenessActionRetryCount() {
        return livenessActionRetryCount;
    }

    public void setLivenessActionRetryCount(Integer livenessActionRetryCount) {
        this.livenessActionRetryCount = livenessActionRetryCount;
    }

    public Integer getEntryFaceVerify() {
        return entryFaceVerify;
    }

    public void setEntryFaceVerify(Integer entryFaceVerify) {
        this.entryFaceVerify = entryFaceVerify;
    }

    public Integer getEntryLivenessVerify() {
        return entryLivenessVerify;
    }

    public void setEntryLivenessVerify(Integer entryLivenessVerify) {
        this.entryLivenessVerify = entryLivenessVerify;
    }

    public Integer getConstantFaceVerify() {
        return constantFaceVerify;
    }

    public void setConstantFaceVerify(Integer constantFaceVerify) {
        this.constantFaceVerify = constantFaceVerify;
    }

    public Integer getConstantLivenessVerifyCount() {
        return constantLivenessVerifyCount;
    }

    public void setConstantLivenessVerifyCount(Integer constantLivenessVerifyCount) {
        this.constantLivenessVerifyCount = constantLivenessVerifyCount;
    }

    public Integer getConstantLivenessRetryCount() {
        return constantLivenessRetryCount;
    }

    public void setConstantLivenessRetryCount(Integer constantLivenessRetryCount) {
        this.constantLivenessRetryCount = constantLivenessRetryCount;
    }

    public Integer getClientVideoPush() {
        return clientVideoPush;
    }

    public void setClientVideoPush(Integer clientVideoPush) {
        this.clientVideoPush = clientVideoPush;
    }

    public Integer getClientVideoRecord() {
        return clientVideoRecord;
    }

    public void setClientVideoRecord(Integer clientVideoRecord) {
        this.clientVideoRecord = clientVideoRecord;
    }

    public Integer getWxappVideoPush() {
        return wxappVideoPush;
    }

    public void setWxappVideoPush(Integer wxappVideoPush) {
        this.wxappVideoPush = wxappVideoPush;
    }

    public Integer getWxappVideoRecord() {
        return wxappVideoRecord;
    }

    public void setWxappVideoRecord(Integer wxappVideoRecord) {
        this.wxappVideoRecord = wxappVideoRecord;
    }

    public Integer getCameraPhotoUpload() {
        return cameraPhotoUpload;
    }

    public void setCameraPhotoUpload(Integer cameraPhotoUpload) {
        this.cameraPhotoUpload = cameraPhotoUpload;
    }

    public Integer getWxappPhotoUpload() {
        return wxappPhotoUpload;
    }

    public void setWxappPhotoUpload(Integer wxappPhotoUpload) {
        this.wxappPhotoUpload = wxappPhotoUpload;
    }

    public Integer getReexamAuditing() {
        return reexamAuditing;
    }

    public void setReexamAuditing(Integer reexamAuditing) {
        this.reexamAuditing = reexamAuditing;
    }

    public Integer getSelectStrategy() {
        return selectStrategy;
    }

    public void setSelectStrategy(Integer selectStrategy) {
        this.selectStrategy = selectStrategy;
    }

    public String getIpLimit() {
        return ipLimit;
    }

    public void setIpLimit(String ipLimit) {
        this.ipLimit = ipLimit;
    }

    public Integer getShowObjectiveScore() {
        return showObjectiveScore;
    }

    public void setShowObjectiveScore(Integer showObjectiveScore) {
        this.showObjectiveScore = showObjectiveScore;
    }

    public Date getCreateTime() {
        return createTime;
    }

    public void setCreateTime(Date createTime) {
        this.createTime = createTime;
    }

    public Date getUpdateTime() {
        return updateTime;
    }

    public void setUpdateTime(Date updateTime) {
        this.updateTime = updateTime;
    }
}
